#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "SteamPlugin.h"


#pragma comment(lib, "steam_api.lib")


//////////////////////////////////////////////////////////////////////////
// this constructor is called when the WME script creates a new object
SteamPlugin::SteamPlugin(IWmeObject* Owner, IWmeGame* Game, IWmeScript* Script, IWmeStack* Stack) :
	m_CallbackUserStatsReceived(this, &SteamPlugin::OnUserStatsReceived),
	m_CallbackUserStatsStored(this, &SteamPlugin::OnUserStatsStored),
	m_CallbackAchievementStored(this, &SteamPlugin::OnAchievementStored)
{
	this->Game = Game;
	m_Owner = Owner;
	
	Stack->ExpectParams(0);
	Game->SubscribeEvent(m_Owner, WME_EVENT_UPDATE);

	InitSteam();
}


//////////////////////////////////////////////////////////////////////////
// this constructor is called when WME restores objects from a saved game
SteamPlugin::SteamPlugin(IWmeObject* Owner, IWmeGame* Game) :
	m_CallbackUserStatsReceived(this, &SteamPlugin::OnUserStatsReceived),
	m_CallbackUserStatsStored(this, &SteamPlugin::OnUserStatsStored),
	m_CallbackAchievementStored(this, &SteamPlugin::OnAchievementStored)
{
	this->Game = Game;
	InitSteam();
}

//////////////////////////////////////////////////////////////////////////
// destructor
SteamPlugin::~SteamPlugin(void)
{
	SteamAPI_Shutdown();
	m_SteamInitialized = false;
}

//////////////////////////////////////////////////////////////////////////
// we call this when we receive an update event from WME
bool SteamPlugin::Update()
{
	if (m_SteamInitialized) SteamAPI_RunCallbacks();
	return true;
}


//////////////////////////////////////////////////////////////////////////
// this method is called whenever the script invokes a method of our object
bool SteamPlugin::CallMethod(const char* MethodName, IWmeScript* Script, IWmeStack* Stack)
{
	//////////////////////////////////////////////////////////////////////////
	// RequestStats()
	//////////////////////////////////////////////////////////////////////////
	if (strcmp(MethodName, "RequestStats") == 0)
	{
		Stack->ExpectParams(0);
		Stack->PushValue(RequestStats());
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// SetAchievement(string id)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "SetAchievement") == 0)
	{
		Stack->ExpectParams(1);
		const char* id = Stack->PopValue()->GetValString();

		Stack->PushValue(SetAchievement(id));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// IsAchieved(string id)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "IsAchieved") == 0)
	{
		Stack->ExpectParams(1);
		const char* id = Stack->PopValue()->GetValString();

		Stack->PushValue(IsAchieved(id));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// ClearAchievement(string id)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "ClearAchievement") == 0)
	{
		Stack->ExpectParams(1);
		const char* id = Stack->PopValue()->GetValString();

		Stack->PushValue(ClearAchievement(id));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// GetAchievementId(int index)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "GetAchievementId") == 0)
	{
		Stack->ExpectParams(1);
		int index = Stack->PopValue()->GetValInt();

		Stack->PushValue(GetAchievementId(index));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// SetStat(string id, int|float value)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "SetStat") == 0)
	{
		Stack->ExpectParams(2);
		const char* id = Stack->PopValue()->GetValString();
		IWmeValue* val = Stack->PopValue();

		if (val->IsValFloat()) Stack->PushValue(SetStat(id, (float)val->GetValFloat()));
		else Stack->PushValue(SetStat(id, val->GetValInt()));

		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// GetStatInt(string id)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "GetStatInt") == 0)
	{
		Stack->ExpectParams(1);
		const char* id = Stack->PopValue()->GetValString();

		Stack->PushValue(GetStatInt(id));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// GetStatFloat(string id)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "GetStatFloat") == 0)
	{
		Stack->ExpectParams(1);
		const char* id = Stack->PopValue()->GetValString();

		Stack->PushValue(GetStatFloat(id));
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// ResetAllStats(bool includingAchievements)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(MethodName, "ResetAllStats") == 0)
	{
		Stack->ExpectParams(1);
		bool includingAchievements = Stack->PopValue()->GetValBool();

		Stack->PushValue(ResetAllStats(includingAchievements));
		return true;
	}

	return false;
}


//////////////////////////////////////////////////////////////////////////
// this method is called whenever the WME script queries a property of our object
bool SteamPlugin::GetProperty(const char* Name, IWmeValue* Value)
{
	//////////////////////////////////////////////////////////////////////////
	// SteamAvailable (RO)
	//////////////////////////////////////////////////////////////////////////
	if (strcmp(Name, "SteamAvailable") == 0)
	{
		Value->SetVal(m_SteamInitialized);
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// StatsAvailable (RO)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(Name, "StatsAvailable") == 0)
	{
		Value->SetVal(m_StatsInitialized);
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// NumAchievements (RO)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(Name, "NumAchievements") == 0)
	{
		Value->SetVal(GetNumAchievements());
		return true;
	}
	//////////////////////////////////////////////////////////////////////////
	// AppId (RO)
	//////////////////////////////////////////////////////////////////////////
	else if (strcmp(Name, "AppId") == 0)
	{
		Value->SetVal(m_AppId);
		return true;
	}

	return false;
}

//////////////////////////////////////////////////////////////////////////
// this method is called whenever the WME script sets a property of our object
bool SteamPlugin::SetProperty(const char* Name, IWmeValue* Value)
{
	return false;
}

//////////////////////////////////////////////////////////////////////////
// this method is called when WME is saving or restoring game
bool SteamPlugin::Serialize(IWmeSerialMgr* SerialMgr)
{
	SerialMgr->TransferValue(&Game);
	SerialMgr->TransferValue(&m_Owner);

	return true;
}

//////////////////////////////////////////////////////////////////////////
// this method is called when any of the subscribed events occured
bool SteamPlugin::ReceiveEvent(EWmeEvent Event, void* EventData1, void* EventData2)
{
	switch(Event)
	{
	case WME_EVENT_UPDATE:
		Update();
		return true;
	}	
	return false;
}


//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::InitSteam()
{
	m_AppId = 0;
	m_StatsInitialized = false;

	m_SteamInitialized = SteamAPI_Init();
	if (!m_SteamInitialized) return false;

	m_AppId = SteamUtils()->GetAppID();

	return RequestStats();
}

//////////////////////////////////////////////////////////////////////////
void SteamPlugin::OnUserStatsReceived(UserStatsReceived_t* pCallback)
{
	if (pCallback->m_nGameID == m_AppId)
	{
		if (pCallback->m_eResult == k_EResultOK)
		{
			m_StatsInitialized = true;
		}
	}
}

//////////////////////////////////////////////////////////////////////////
void SteamPlugin::OnUserStatsStored(UserStatsStored_t* pCallback)
{
	// we may get callbacks for other games' stats arriving, ignore them
	if (pCallback->m_nGameID == m_AppId)
	{
		if (pCallback->m_eResult == k_EResultOK)
		{
			OutputDebugString("Stored stats for Steam\n");
		}
		else
		{
			char buffer[128];
			_snprintf(buffer, 128, "StatsStored - failed, %d\n", pCallback->m_eResult);
			OutputDebugString(buffer);
		}
	}
}

//////////////////////////////////////////////////////////////////////////
void SteamPlugin::OnAchievementStored(UserAchievementStored_t* pCallback)
{
	// we may get callbacks for other games' stats arriving, ignore them
	if (pCallback->m_nGameID == m_AppId)
	{
		OutputDebugString( "Stored Achievement for Steam\n" );
	}
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::RequestStats()
{
	// Is Steam loaded? If not we can't get stats.
	if (SteamUserStats() == NULL || SteamUser() == NULL)
	{
		return false;
	}
	// Is the user logged on?  If not we can't get stats.
	if (!SteamUser()->BLoggedOn())
	{
		return false;
	}
	// Request user stats.
	return SteamUserStats()->RequestCurrentStats();
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::SetAchievement(const char* id)
{
	if (!m_StatsInitialized) return false;

	if (!SteamUserStats()->SetAchievement(id)) return false;
	return SteamUserStats()->StoreStats();
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::IsAchieved(const char* id)
{
	if (!m_StatsInitialized) return false;

	bool achieved;
	SteamUserStats()->GetAchievement(id, &achieved);

	return achieved;
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::ClearAchievement(const char* id)
{
	if (!m_StatsInitialized) return false;
	else
	{
		if (!SteamUserStats()->ClearAchievement(id)) return false;
		return SteamUserStats()->StoreStats();
	}
}

//////////////////////////////////////////////////////////////////////////
int SteamPlugin::GetNumAchievements()
{
	if (!m_StatsInitialized) return 0;
	return SteamUserStats()->GetNumAchievements();
}

//////////////////////////////////////////////////////////////////////////
const char* SteamPlugin::GetAchievementId(int index)
{
	if (!m_StatsInitialized) return 0;
	return SteamUserStats()->GetAchievementName(index);
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::SetStat(const char* id, int value)
{
	if (!m_StatsInitialized) return false;

	if (!SteamUserStats()->SetStat(id, value)) return false;
	return SteamUserStats()->StoreStats();
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::SetStat(const char* id, float value)
{
	if (!m_StatsInitialized) return false;

	if (!SteamUserStats()->SetStat(id, value)) return false;
	return SteamUserStats()->StoreStats();
}

//////////////////////////////////////////////////////////////////////////
int SteamPlugin::GetStatInt(const char* id)
{
	if (!m_StatsInitialized) return 0;

	int value;
	if (!SteamUserStats()->GetStat(id, &value)) return 0;
	else return value;
}

//////////////////////////////////////////////////////////////////////////
float SteamPlugin::GetStatFloat(const char* id)
{
	if (!m_StatsInitialized) return 0.0f;

	float value;
	if (!SteamUserStats()->GetStat(id, &value)) return 0.0f;
	else return value;
}

//////////////////////////////////////////////////////////////////////////
bool SteamPlugin::ResetAllStats(bool includingAchievements)
{
	if (!m_StatsInitialized) return false;

	return SteamUserStats()->ResetAllStats(includingAchievements);
}
